/**
 * 
 */
package com.wieseke.cptk.common.dao;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import com.wieseke.cptk.common.constants.EventConstants;
import com.wieseke.cptk.common.util.BigDecimalUtil;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ReconstructionCosts {

	/**
	 * map of the event costs
	 */
	private Map<String, BigDecimal> costs;

	/**
	 * @param cospeciations
	 * @param sortings
	 * @param duplications
	 * @param hostswitches
	 */
	public ReconstructionCosts(BigDecimal cospeciationCosts,
			BigDecimal sortingCosts, BigDecimal duplicationCosts,
			BigDecimal hostswitchCosts) {
		this.costs = new HashMap<String, BigDecimal>();
		this.costs.put(EventConstants.COSPECIATION, cospeciationCosts);
		this.costs.put(EventConstants.SORTING, sortingCosts);
		this.costs.put(EventConstants.DUPLICATION, duplicationCosts);
		this.costs.put(EventConstants.HOSTSWITCH, hostswitchCosts);
	}

	public ReconstructionCosts(double[] costs) {
		this(BigDecimal.valueOf(costs[0]), BigDecimal.valueOf(costs[1]),
				BigDecimal.valueOf(costs[2]), BigDecimal.valueOf(costs[3]));
	}

	public ReconstructionCosts(ReconstructionCosts costs) {
		this.costs = new HashMap<String, BigDecimal>();
		this.costs.put(EventConstants.COSPECIATION, BigDecimalUtil.copy(costs
				.getCospeciationCosts()));
		this.costs.put(EventConstants.SORTING, BigDecimalUtil.copy(costs
				.getSortingCosts()));
		this.costs.put(EventConstants.DUPLICATION, BigDecimalUtil.copy(costs
				.getDuplicationCosts()));
		this.costs.put(EventConstants.HOSTSWITCH, BigDecimalUtil.copy(costs
				.getHostswitchCosts()));
	}

	public ReconstructionCosts(Map<String, BigDecimal> costs) {
		this.costs = new HashMap<String, BigDecimal>();
		this.costs.put(EventConstants.COSPECIATION, BigDecimalUtil.copy(costs
				.get(EventConstants.COSPECIATION)));
		this.costs.put(EventConstants.SORTING, BigDecimalUtil.copy(costs
				.get(EventConstants.SORTING)));
		this.costs.put(EventConstants.DUPLICATION, BigDecimalUtil.copy(costs
				.get(EventConstants.DUPLICATION)));
		this.costs.put(EventConstants.HOSTSWITCH, BigDecimalUtil.copy(costs
				.get(EventConstants.HOSTSWITCH)));
	}

	public ReconstructionCosts() {
		this.costs = new HashMap<String, BigDecimal>();
	}

	/**
	 * @return the cospeciations
	 */
	public BigDecimal getCospeciationCosts() {
		return this.costs.get(EventConstants.COSPECIATION);
	}

	/**
	 * @param cospeciationCosts
	 *            the cospeciationCosts to set
	 */
	public void setCospeciationCosts(BigDecimal cospeciationCosts) {
		this.costs.put(EventConstants.COSPECIATION, cospeciationCosts);
	}

	/**
	 * @return the sortings
	 */
	public BigDecimal getSortingCosts() {
		return this.costs.get(EventConstants.SORTING);
	}

	/**
	 * @param sortingCosts
	 *            the sortingCosts to set
	 */
	public void setSortingCosts(BigDecimal sortingCosts) {
		this.costs.put(EventConstants.SORTING, sortingCosts);
	}

	/**
	 * @return the duplicationCosts
	 */
	public BigDecimal getDuplicationCosts() {
		return this.costs.get(EventConstants.DUPLICATION);
	}

	/**
	 * @param duplicationCosts
	 *            the duplicationCosts to set
	 */
	public void setDuplicationCosts(BigDecimal duplicationCosts) {
		this.costs.put(EventConstants.DUPLICATION, duplicationCosts);
	}

	/**
	 * @return the hostswitchCosts
	 */
	public BigDecimal getHostswitchCosts() {
		return this.costs.get(EventConstants.HOSTSWITCH);
	}

	/**
	 * @param hostswitchCosts
	 *            the hostswitchCosts to set
	 */
	public void setHostswitchCosts(BigDecimal hostswitchCosts) {
		this.costs.put(EventConstants.HOSTSWITCH, hostswitchCosts);
	}

	/**
	 * @return the events
	 */
	public Map<String, BigDecimal> getCosts() {
		return costs;
	}

	/**
	 * @param events
	 *            the events to set
	 */
	public void setCosts(Map<String, BigDecimal> events) {
		this.costs = events;
	}

	/**
	 * @return the costs
	 */
	public double[] getDoubleCosts() {
		double[] result = new double[4];
		result[0] = getCospeciationCosts().doubleValue();
		result[1] = getSortingCosts().doubleValue();
		result[2] = getDuplicationCosts().doubleValue();
		result[3] = getHostswitchCosts().doubleValue();
		return result;
	}

	/**
	 * @return a copy of the ReconstructionCosts object
	 */
	public ReconstructionCosts copy() {
		return new ReconstructionCosts(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((costs == null) ? 0 : costs.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ReconstructionCosts))
			return false;
		ReconstructionCosts other = (ReconstructionCosts) obj;
		if (costs == null) {
			if (other.costs != null)
				return false;
		} else if (!costs.equals(other.costs))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */

	public String toString() {
		return "(" + this.getCospeciationCosts() + " " + this.getSortingCosts()
				+ " " + this.getDuplicationCosts() + " "
				+ this.getHostswitchCosts() + ")";
	}

	public Collection<BigDecimal> values() {
		return this.costs.values();
	}

	public BigDecimal get(Object key) {
		return this.costs.get(key);
	}

	public BigDecimal put(String key, BigDecimal value) {
		return this.costs.put(key, value);
	}

	public Set<Entry<String, BigDecimal>> entrySet() {
		return this.costs.entrySet();
	}
}
