/**
 * 
 */
package com.wieseke.cptk.common.view;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import com.wieseke.cptk.common.provider.NodePropertiesContentProvider;
import com.wieseke.cptk.common.provider.NodePropertiesLabelProvider;
import com.wieseke.cptk.input.model.InputNodeProperties;
import com.wieseke.cptk.input.selection.InputNodeControlSelection;
import com.wieseke.cptk.input.viewer.InputNodeControl;
import com.wieseke.cptk.output.model.OutputNodeProperties;
import com.wieseke.cptk.output.selection.OutputNodeControlSelection;
import com.wieseke.cptk.output.viewer.OutputNodeControl;

/**
 * @author Nic
 * 
 */
public class NodePropertiesView extends ViewPart {

	public static final String ID = "com.wieseke.cptk.common.view.NodePropertiesView";

	private TableViewer viewer;
	private TableColumn nameColumn;
	private TableColumn valueColumn;

	private ISelectionListener nodeSelectionListener;

	/**
	 * 
	 */
	public NodePropertiesView() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		final Table table = viewer.getTable();

		this.nameColumn = new TableColumn(table, SWT.LEFT);
		this.nameColumn.setText("Property");
		this.nameColumn.setWidth(150);

		this.valueColumn = new TableColumn(table, SWT.LEFT);
		this.valueColumn.setText("Value");
		this.valueColumn.setWidth(300);

		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		viewer.setContentProvider(new NodePropertiesContentProvider(viewer));
		viewer.setLabelProvider(new NodePropertiesLabelProvider());
		// getSite().setSelectionProvider(viewer);
		hookNodeSelection();
	}

	private void hookNodeSelection() {
		this.nodeSelectionListener = new ISelectionListener() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see
			 * org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse
			 * .ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
			 */
			
			public void selectionChanged(IWorkbenchPart part,
					ISelection selection) {
				nodeSelectionChanged(part, selection);

			}
		};
		getSite().getPage().addSelectionListener(this.nodeSelectionListener);
	}

	protected void nodeSelectionChanged(IWorkbenchPart part,
			ISelection selection) {
		if (part == this)
			return;
		if (selection instanceof InputNodeControlSelection) {
			InputNodeControl nodeControl = ((InputNodeControlSelection) selection)
					.getInputNodeControl();
			if (nodeControl == null) {
				this.viewer.setInput(null);
				return;
			}
			InputNodeProperties nodeInfo = new InputNodeProperties(nodeControl
					.getNode());
			this.viewer.setInput(nodeInfo);
		} else if (selection instanceof OutputNodeControlSelection) {
			OutputNodeControl nodeControl = ((OutputNodeControlSelection) selection)
					.getOutputNodeControl();
			if (nodeControl == null) {
				this.viewer.setInput(null);
				return;
			}
			OutputNodeProperties nodeInfo = new OutputNodeProperties(
					nodeControl.getNode());
			this.viewer.setInput(nodeInfo);
		} else {
			this.viewer.setInput(null);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	
	public void dispose() {
		if (this.nodeSelectionListener != null) {
			getSite().getPage().removeSelectionListener(
					this.nodeSelectionListener);
		}
		super.dispose();
	}
}
