/**
 * 
 */
package com.wieseke.cptk.input.dao;

import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.dao.PositionNode;
import com.wieseke.cptk.common.util.Tupel;
import com.wieseke.cptk.input.preferences.InputPreferenceConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public abstract class InputNode extends PositionNode {

	public void setParent(InputNode parent) {
		super.setParent(parent);
	}

	@SuppressWarnings("unchecked")
	
	public List<? extends InputNode> getChildren() {
		return (List<InputNode>) super.getChildren();
	}

	/**
	 * calculates the positions for the descending nodes
	 * 
	 * @param nextLeafPosX
	 *            the x position of the next leaf
	 * @return a tupel containing the nextLeafPosX ant the nextPosY
	 */
	public Tupel<Integer, Integer> calcNodePositions(int nextLeafPosX) {
		IPreferenceStore store = PlatformUI.getPreferenceStore();
		int xDistance = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_REARRANGE_NODES_XDISTANCE);
		int yDistance = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_REARRANGE_NODES_YDISTANCE);
		String linetype = store
				.getString(InputPreferenceConstants.INPUT_VIEWER_NAME_LINETYPE);

		int nextPosY = 0;
		if (isLeaf()) {
			setPosX(nextLeafPosX * xDistance);
			nextLeafPosX++;
			nextPosY++;
			setPosY(nextPosY * yDistance);
		} else {
			for (InputNode node : getChildren()) {
				Tupel<Integer, Integer> integerTupel = ((InputNode) node)
						.calcNodePositions(nextLeafPosX);
				nextLeafPosX = integerTupel.getValue1();
				nextPosY = Math.max(nextPosY, integerTupel.getValue2());
			}

			InputNode leftLeaf;
			int leftChildPosX;
			InputNode rightLeaf;
			int rightChildPosX;

			// diagonal lines
			if (linetype
					.equals(InputPreferenceConstants.INPUT_VIEWER_LINETYPE_ACUTE_ANGLED)) {
				leftLeaf = this;
				rightLeaf = this;
				while (!leftLeaf.isLeaf()) {
					leftLeaf = leftLeaf.getChildren().get(0);
				}
				while (!rightLeaf.isLeaf()) {
					rightLeaf = rightLeaf.getChildren().get(
							rightLeaf.getDirectChildrenCount() - 1);
				}
				int minPosY = yDistance / 2;
				for (InputNode child : this.getChildren()) {
					minPosY = Math
							.max(minPosY, child.getPosY() + yDistance / 2);
				}

				leftChildPosX = leftLeaf.getPosX();
				rightChildPosX = rightLeaf.getPosX();

				setPosX((rightChildPosX + leftChildPosX) / 2);
				setPosY(Math.max(minPosY, yDistance / 2
						* Math.abs(rightChildPosX - leftChildPosX) / xDistance
						+ yDistance));
			}
			// orthogonal lines
			else {
				// set y position of the child node as high as possible (except
				// for the leaf nodes)
				// for (InputNode node : getChildren()) {
				// if (!node.isLeaf())
				// node.setPosY(nextPosY * yDistance);
				// }

				leftLeaf = getChildren().get(0);
				rightLeaf = getChildren().get(getChildren().size() - 1);

				leftChildPosX = leftLeaf.getPosX();
				rightChildPosX = rightLeaf.getPosX();

				nextPosY++;

				setPosX((rightChildPosX + leftChildPosX) / 2);
				setPosY(nextPosY * yDistance);
			}

		}
		return new Tupel<Integer, Integer>(nextLeafPosX, nextPosY);
	}

	public int renumberNodes(int number) {
		this.setNumber(number);
		number++;
		for (InputNode child : getChildren()) {
			number = child.renumberNodes(number);
		}
		this.setChildrenCount(number - this.getNumber() - 1);
		return number;
	}

	public void renameNodes() {
		String label = ((this instanceof InputHostNode) ? "H" : "P")
				+ this.getNumber();
		this.setLabel(label);
		for (InputNode child : getChildren()) {
			child.renameNodes();
		}
	}

}
