/**
 * 
 */
package com.wieseke.cptk.input.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputHostNodeControl;
import com.wieseke.cptk.input.viewer.InputParasiteNodeControl;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditAddParasiteAssociationOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputParasiteNode parasiteNode;
	private InputHostNode hostNode;
	private boolean associationExisted;

	public EditAddParasiteAssociationOperation(InputCophylogenyViewer viewer,
			InputParasiteNode node) {
		super("add association to node " + node.getLabel());
		this.viewer = viewer;
		this.parasiteNode = node;
		this.associationExisted = false;
		if (this.viewer.getPhiSource() != null
				&& this.viewer.getPhiSource() instanceof InputHostNodeControl) {
			this.hostNode = (InputHostNode) this.viewer.getPhiSource()
					.getNode();
			if (this.parasiteNode.getAssociations().contains(this.hostNode)) {
				this.associationExisted = true;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */

	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		if (this.viewer.getPhiSource() == null
				|| this.viewer.getPhiSource() instanceof InputParasiteNodeControl) {
			this.viewer.setPhiSource(null);
			return Status.CANCEL_STATUS;
		}
		return redo(monitor, info);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */

	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		if (this.associationExisted) {
			this.parasiteNode.getAssociations().remove(this.hostNode);
		} else {
			this.parasiteNode.getAssociations().add(this.hostNode);
		}
		this.viewer.setPhiSource(null);
		this.viewer.updateAssociations();
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */

	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		if (this.associationExisted) {
			this.parasiteNode.getAssociations().add(this.hostNode);
		} else {
			this.parasiteNode.getAssociations().remove(this.hostNode);
		}
		this.viewer.setPhiSource(null);
		this.viewer.updateAssociations();
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
