/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.input.dao.InputCophylogeny;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ChangeCophylogenyPropertiesWizard extends Wizard {

	private ChooseCophylogenyCostsWizardPage chooseCophylogenyCostsWizardPage;
	private ChooseCophylogenyOptionsWizardPage chooseCophylogenyOptionsWizardPage;
	private InputCophylogeny cophylogeny;

	public ChangeCophylogenyPropertiesWizard(InputCophylogeny cophylogeny) {
		this.cophylogeny = cophylogeny;
		setWindowTitle("Reconstruction Properties");
	}

	
	public boolean performFinish() {
		Map<String, Text> texts = chooseCophylogenyOptionsWizardPage.getTexts();
		for (Entry<String, Text> entry : texts.entrySet()) {
			if (!OptionsConstants.checkOption(entry.getKey(), entry.getValue().getText())) {
				String message = "Error parsing the option " + entry.getKey() + ".";
				chooseCophylogenyCostsWizardPage.setMessage(message);
				chooseCophylogenyOptionsWizardPage.setMessage(message);
				return false;
			}
		}

		ReconstructionCosts costs = new ReconstructionCosts();
		Map<String, String> options = new HashMap<String, String>();
		if (chooseCophylogenyCostsWizardPage.useProbabilisticValues()) {
			for (Entry<String, Scale> entry : chooseCophylogenyCostsWizardPage.getPScales().entrySet()) {
				BigDecimal value = chooseCophylogenyCostsWizardPage.transformFromControlValue(entry.getValue()
						.getSelection());
				costs.put(entry.getKey(), value);
			}
			options.put(OptionsConstants.PROBABILITY_COSTS_ID, OptionsConstants.PROBABILITY_COSTS_1_ID);
		} else {
			for (Entry<String, Text> entry : chooseCophylogenyCostsWizardPage.getATexts().entrySet()) {
				BigDecimal value = BigDecimal.ZERO;
				try {
					value = new BigDecimal(entry.getValue().getText());
				} catch (NumberFormatException e) {
					String message = "\"" + entry.getValue().getText() + "\" is no valid input for the "
							+ entry.getKey().toLowerCase() + "costs.";
					chooseCophylogenyCostsWizardPage.setMessage(message);
					chooseCophylogenyOptionsWizardPage.setMessage(message);
					return false;
				}
				costs.put(entry.getKey(), value);
			}
			options.put(OptionsConstants.PROBABILITY_COSTS_ID, OptionsConstants.PROBABILITY_COSTS_0_ID);
		}

		for (Entry<String, Combo> entry : chooseCophylogenyOptionsWizardPage.getCombos().entrySet()) {
			String value = String.valueOf(entry.getValue().getSelectionIndex());
			options.put(entry.getKey(), value);
		}
		for (Entry<String, Text> entry : chooseCophylogenyOptionsWizardPage.getTexts().entrySet()) {
			String value = entry.getValue().getText();
			options.put(entry.getKey(), value);
		}

		this.cophylogeny.setCosts(costs);
		this.cophylogeny.setOptions(options);

		return true;
	}

	
	public void addPages() {
		chooseCophylogenyCostsWizardPage = new ChooseCophylogenyCostsWizardPage("Change Costs", this.cophylogeny);
		addPage(chooseCophylogenyCostsWizardPage);
		chooseCophylogenyOptionsWizardPage = new ChooseCophylogenyOptionsWizardPage("Change Options", this.cophylogeny);
		addPage(chooseCophylogenyOptionsWizardPage);
		super.addPages();
	}

}
