/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.util.Tupel;
import com.wieseke.cptk.input.dao.InputCophylogeny;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ChooseCophylogenyOptionsWizardPage extends WizardPage {

	private InputCophylogeny cophylogeny;

	private Group optionsGroup;
	private Map<String, Combo> combos;
	private Map<String, Text> texts;

	private ScrolledComposite scrolledComposite;
	private Composite container;

	public ChooseCophylogenyOptionsWizardPage(String pageName,
			InputCophylogeny cophylogeny) {
		super(pageName);
		this.cophylogeny = cophylogeny;
		this.combos = new HashMap<String, Combo>();
		this.texts = new HashMap<String, Text>();
		this.setTitle(pageName);
		this.setDescription("Choose the reconstruction options.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);

		FillLayout compositeLayout = new FillLayout();
		compositeLayout.spacing = 10;
		compositeLayout.marginWidth = 10;
		compositeLayout.marginHeight = 10;
		composite.setLayout(compositeLayout);

		optionsGroup = new Group(composite, SWT.NONE);
		FillLayout optionsGroupLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		optionsGroupLayout.marginHeight = 10;
		optionsGroupLayout.marginWidth = 10;
		optionsGroupLayout.spacing = 10;
		optionsGroup.setLayout(optionsGroupLayout);
		optionsGroup.setText("Options");

		scrolledComposite = new ScrolledComposite(optionsGroup, SWT.H_SCROLL
				| SWT.V_SCROLL);
		container = new Composite(scrolledComposite, SWT.NONE);
		scrolledComposite.addControlListener(new ControlListener() {
			
			public void controlMoved(ControlEvent e) {
			}

			
			public void controlResized(ControlEvent e) {
				Point containerSize = container.computeSize(SWT.DEFAULT,
						SWT.DEFAULT);
				Point scrolledContainerSize = scrolledComposite.getSize();
				if (scrolledComposite.getVerticalBar().isVisible())
					scrolledContainerSize.x -= scrolledComposite
							.getVerticalBar().getSize().x;
				if (scrolledComposite.getHorizontalBar().isVisible())
					scrolledContainerSize.y -= scrolledComposite
							.getHorizontalBar().getSize().y;
				container.setSize(new Point(Math.max(containerSize.x,
						scrolledContainerSize.x), Math.max(containerSize.y,
						scrolledContainerSize.y)));
			}

		});
		scrolledComposite.setContent(container);
		GridLayout containerGridLayout = new GridLayout(2, false);
		container.setLayout(containerGridLayout);

		Map<String, Tupel<String, Map<String, String>>> options = OptionsConstants
				.getOptions();

		for (Entry<String, Tupel<String, Map<String, String>>> option : options
				.entrySet()) {

			Tupel<String, Map<String, String>> optionsEntry = option.getValue();
			Label label = new Label(container, SWT.NONE);
			GridData labelLayoutData = new GridData();
			labelLayoutData.widthHint = 150;
			label.setLayoutData(labelLayoutData);
			label.setText(option.getKey());
			label.setToolTipText(optionsEntry.getValue1());

			if (optionsEntry.getValue2() == null) {
				Text text = new Text(container, SWT.BORDER);
				GridData textLayoutData = new GridData();
				textLayoutData.grabExcessHorizontalSpace = true;
				textLayoutData.horizontalAlignment = GridData.FILL;
				text.setLayoutData(textLayoutData);
				text.setText(cophylogeny.getOptions().get(option.getKey()));
				this.texts.put(option.getKey(), text);
			} else {
				Combo combo = new Combo(container, SWT.DROP_DOWN
						| SWT.READ_ONLY);
				GridData comboLayoutData = new GridData();
				comboLayoutData.grabExcessHorizontalSpace = true;
				comboLayoutData.horizontalAlignment = GridData.FILL;
				combo.setLayoutData(comboLayoutData);
				for (Entry<String, String> entry : optionsEntry.getValue2()
						.entrySet()) {
					combo.add(entry.getValue());
				}
				try {
					int index = Integer.parseInt(this.cophylogeny.getOptions()
							.get(option.getKey()));
					combo.getItem(index);
					combo.select(index);
				} catch (Exception e) {
					combo.select(0);
				}
				this.combos.put(option.getKey(), combo);
			}
		}

		container.setSize(container.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		composite.layout();

		setControl(composite);
	}

	/**
	 * @return the combos
	 */
	public Map<String, Combo> getCombos() {
		return combos;
	}

	/**
	 * @return the texts
	 */
	public Map<String, Text> getTexts() {
		return texts;
	}

}
