/**
 * 
 */
package com.wieseke.cptk.output.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.texteditor.InfoForm;

import com.wieseke.cptk.common.action.CommonAction;
import com.wieseke.cptk.common.action.CophylogenyViewerActionFactory;
import com.wieseke.cptk.output.action.OutputAction;
import com.wieseke.cptk.output.action.OutputCophylogenyViewerActionFactory;
import com.wieseke.cptk.output.action.OutputNodeControlActionFactory;
import com.wieseke.cptk.output.constants.OutputViewerConstants;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.output.viewer.OutputCophylogenyViewer;

/**
 * @author Nicolas Wieseke
 * 
 */
@SuppressWarnings("deprecation")
public class VisualOutputEditor extends EditorPart {

	public static final String ID = "com.wieseke.cptk.output.editor.VisualOutputEditor";

	private OutputCophylogenyViewer outputCophylogenyViewer;
	private boolean dirty;

	private List<CommonAction> exportActions = new ArrayList<CommonAction>();
	private List<OutputAction> nodeActions = new ArrayList<OutputAction>();
	private List<CommonAction> zoomActions = new ArrayList<CommonAction>();
	private List<OutputAction> selectEditModeActions = new ArrayList<OutputAction>();
	private List<CommonAction> nodesActions = new ArrayList<CommonAction>();
	private List<CommonAction> treeActions = new ArrayList<CommonAction>();
	private List<OutputAction> reconstructionActions = new ArrayList<OutputAction>();

	/**
	 * default constructor
	 */
	public VisualOutputEditor() {
		this.outputCophylogenyViewer = null;
		this.dirty = false;
	}

	
	public String getTitle() {
		if (getEditorInput() instanceof VisualOutputEditorInput) {
			return ((VisualOutputEditorInput) getEditorInput()).getName();
		}
		return super.getTitle();
	}

	
	public String getPartName() {
		if (getEditorInput() instanceof VisualOutputEditorInput) {
			return ((VisualOutputEditorInput) getEditorInput()).getName();
		}
		return super.getPartName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	
	public void doSave(IProgressMonitor monitor) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	
	public void doSaveAs() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite,
	 * org.eclipse.ui.IEditorInput)
	 */
	
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	
	public boolean isDirty() {
		return false; // dirty;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	
	public boolean isSaveAsAllowed() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	
	public void createPartControl(Composite parent) {

		try {
			// InputStream inputStream;
			// String filename = getEditorInput().getName();
			// if (getEditorInput() instanceof FileStoreEditorInput) {
			// URI uri = ((FileStoreEditorInput) getEditorInput()).getURI();
			// inputStream = FileUtils.openInputStream(new File(uri));
			// } else {
			// IFile file =
			// org.eclipse.ui.ide.ResourceUtil.getFile(getEditorInput());
			// inputStream = file.getContents();
			// }
			//
			// NexusDeserializer nexusDeserializer = new NexusDeserializer();
			//
			// OutputCophylogeny cophylogeny =
			// nexusDeserializer.deserialize(inputStream, filename);
			if (getEditorInput() instanceof VisualOutputEditorInput) {
				VisualOutputEditorInput input = (VisualOutputEditorInput) getEditorInput();
				OutputCophylogeny cophylogeny = new OutputCophylogeny(input
						.getCophylogeny());

				this.outputCophylogenyViewer = new OutputCophylogenyViewer(
						parent, cophylogeny, getEditorInput().getName());
				getSite().setSelectionProvider(this.outputCophylogenyViewer);
				this.outputCophylogenyViewer
						.addPropertyChangeListener(new IPropertyChangeListener() {

							
							public void propertyChange(PropertyChangeEvent event) {
								if (event.getProperty().equals(
										OutputViewerConstants.PROPERTY_DIRTY)
										&& !isDirty()) {
									if (dirty != (Boolean) event.getNewValue()) {
										dirty = (Boolean) event.getNewValue();
										firePropertyChange(IEditorPart.PROP_DIRTY);
									}
								}

							}

						});
				createActions();
				createNodeContextMenu();
				createViewerContextMenu();
				getSite().setSelectionProvider(outputCophylogenyViewer);
				ISelectionService service = (ISelectionService) getSite()
						.getService(ISelectionService.class);
				service.addSelectionListener(new ISelectionListener() {
					
					public void selectionChanged(IWorkbenchPart part,
							ISelection selection) {
					}
				});
			}
		} catch (Exception e) {
			InfoForm infoForm = new InfoForm(parent);
			infoForm.setInfo(e.getMessage());
		}
	}

	private void createActions() {
		OutputNodeControlActionFactory nodeActionsToCreate[] = new OutputNodeControlActionFactory[] {
				OutputNodeControlActionFactory.FOLD_NODE,
				OutputNodeControlActionFactory.CHANGE_LABEL };
		for (OutputNodeControlActionFactory factory : nodeActionsToCreate) {
			OutputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.nodeActions.add(action);
		}
		CophylogenyViewerActionFactory exportActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.PRINT,
				CophylogenyViewerActionFactory.EXPORT_SVG };
		CophylogenyViewerActionFactory zoomActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.ZOOM_RESET,
				CophylogenyViewerActionFactory.ZOOM_IN,
				CophylogenyViewerActionFactory.ZOOM_OUT };
		OutputCophylogenyViewerActionFactory selectEditModeActionsToCreate[] = new OutputCophylogenyViewerActionFactory[] {
				OutputCophylogenyViewerActionFactory.SELECT_FOLD_NODE,
				OutputCophylogenyViewerActionFactory.SELECT_CHANGE_LABEL };
		CophylogenyViewerActionFactory nodesActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				// CophylogenyViewerActionFactory.ADJUST_CHILD_NODES,
				CophylogenyViewerActionFactory.SHOW_HOST_LABELS,
				CophylogenyViewerActionFactory.SHOW_PARASITE_LABELS,
				CophylogenyViewerActionFactory.SHOW_RANK_LABELS
		// ,CophylogenyViewerActionFactory.RECALCULATE_NODE_POSITIONS
		};
		CophylogenyViewerActionFactory treeActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.FLIP_HORIZONTAL,
				CophylogenyViewerActionFactory.FLIP_VERTICAL,
				CophylogenyViewerActionFactory.ROTATE_LEFT,
				CophylogenyViewerActionFactory.ROTATE_RIGHT };
		OutputCophylogenyViewerActionFactory reconstructionActionsToCreate[] = new OutputCophylogenyViewerActionFactory[] {
				OutputCophylogenyViewerActionFactory.RECONSTRUCTION_PREVIOUS,
				OutputCophylogenyViewerActionFactory.RECONSTRUCTION_NEXT,
				OutputCophylogenyViewerActionFactory.RECONSTRUCTION_CHOOSE };
		for (CophylogenyViewerActionFactory factory : exportActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.exportActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : zoomActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.zoomActions.add(action);
		}
		for (OutputCophylogenyViewerActionFactory factory : selectEditModeActionsToCreate) {
			OutputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.selectEditModeActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : nodesActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.nodesActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : treeActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.treeActions.add(action);
		}
		for (OutputCophylogenyViewerActionFactory factory : reconstructionActionsToCreate) {
			OutputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.reconstructionActions.add(action);
		}
	}

	private void createNodeContextMenu() {
		MenuManager menuManager = new MenuManager("#NodePopupMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener() {
			
			public void menuAboutToShow(IMenuManager manager) {
				VisualOutputEditor.this.fillNodeContextMenu(manager);
			}
		});
		Menu menu = menuManager.createContextMenu(this.outputCophylogenyViewer);
		this.outputCophylogenyViewer.setNodeContextMenu(menu);
		getSite()
				.registerContextMenu(menuManager, this.outputCophylogenyViewer);
	}

	private void createViewerContextMenu() {
		MenuManager menuManager = new MenuManager("#ViewerPopupMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener() {
			
			public void menuAboutToShow(IMenuManager manager) {
				VisualOutputEditor.this.fillViewerContextMenu(manager);
			}
		});
		Menu menu = menuManager.createContextMenu(this.outputCophylogenyViewer);
		this.outputCophylogenyViewer.setViewerMenu(menu);
		getSite()
				.registerContextMenu(menuManager, this.outputCophylogenyViewer);
	}

	private void fillNodeContextMenu(IMenuManager menuManager) {
		boolean isEmpty = this.outputCophylogenyViewer.getSelection().isEmpty();
		for (OutputAction action : this.nodeActions) {
			action.setEnabled(!isEmpty);
			menuManager.add(action);
		}
		menuManager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillViewerContextMenu(IMenuManager menuManager) {
		for (CommonAction action : this.exportActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		for (CommonAction action : this.zoomActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		IMenuManager selectEditModeActionMenu = new MenuManager(
				"Select &edit mode");
		for (OutputAction action : this.selectEditModeActions) {
			action.setEnabled(true);
			selectEditModeActionMenu.add(action);
		}
		menuManager.add(selectEditModeActionMenu);
		menuManager.add(new Separator());
		for (CommonAction action : this.nodesActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		for (CommonAction action : this.treeActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		for (OutputAction action : this.reconstructionActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	
	public void setFocus() {
		if (this.outputCophylogenyViewer != null) {
			this.outputCophylogenyViewer.setFocus();
		}
	}

	/**
	 * @return the inputCophylogeny
	 */
	public OutputCophylogeny getInputCophylogeny() {
		return this.outputCophylogenyViewer.getCophylogeny();
	}

	/**
	 * @param inputCophylogeny
	 *            the inputCophylogeny to set
	 */
	public void setInputCophylogeny(OutputCophylogeny inputCophylogeny) {
		this.outputCophylogenyViewer.setCophylogeny(inputCophylogeny);
		this.outputCophylogenyViewer.initCophylogeny(true, null);
	}

	/**
	 * @return the inputCophylogenyComposite
	 */
	public OutputCophylogenyViewer getInputCophylogenyViewer() {
		return outputCophylogenyViewer;
	}

	/**
	 * @param inputCophylogenyViewer
	 *            the inputCophylogenyComposite to set
	 */
	public void setInputCophylogenyViewer(
			OutputCophylogenyViewer inputCophylogenyViewer) {
		this.outputCophylogenyViewer = inputCophylogenyViewer;
	}

	public void rotateRight() {
		this.outputCophylogenyViewer.rotateRight();
	}

	public void rotateLeft() {
		this.outputCophylogenyViewer.rotateLeft();
	}

	public void flipVertical() {
		this.outputCophylogenyViewer.flipVertical();
	}

	public void flipHorizontal() {
		this.outputCophylogenyViewer.flipHorizontal();
	}

	/**
	 * @param dirty
	 *            the dirty to set
	 */
	public void setDirty(boolean dirty) {
		if (this.dirty != dirty) {
			this.dirty = dirty;
			this.outputCophylogenyViewer.setDirty(dirty);
			firePropertyChange(IEditorPart.PROP_DIRTY);
		}
	}

}
