/**
 * 
 */
package com.wieseke.cptk.output.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;

import com.wieseke.cptk.output.dao.OutputNode;
import com.wieseke.cptk.output.viewer.OutputCophylogenyViewer;
import com.wieseke.cptk.output.wizards.ChangeLabelWizard;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditChangeLabelOperation extends AbstractOperation {

	private OutputCophylogenyViewer viewer;
	private OutputNode node;
	private String oldLabel;
	private String newLabel;

	public EditChangeLabelOperation(OutputCophylogenyViewer viewer, OutputNode node) {
		super("change label of node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.oldLabel = node.getLabel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		ChangeLabelWizard wizard = new ChangeLabelWizard(this.node, this.viewer.getCophylogeny());
		WizardDialog dialog = new WizardDialog(this.viewer.getShell(), wizard);
		dialog.create();
		dialog.open();
		if (dialog.getReturnCode() == Window.OK && !this.oldLabel.equals(wizard.getLabel())) {
			this.newLabel = wizard.getLabel();
			return redo(monitor, info);
		}
		return Status.CANCEL_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		this.node.setLabel(this.newLabel);
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		this.node.setLabel(this.oldLabel);
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
