/**
 * 
 */
package com.wieseke.cptk.reconstruction.dao;

import java.util.HashMap;
import java.util.Map;

import com.wieseke.cptk.common.constants.EventConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EventType {

	/**
	 * map of the occurring number of events
	 */
	private Map<String, Integer> events;

	/**
	 * @param cospeciations
	 * @param sortings
	 * @param duplications
	 * @param hostswitches
	 */
	public EventType(int cospeciations, int sortings, int duplications,
			int hostswitches) {
		this.events = new HashMap<String, Integer>();
		this.events.put(EventConstants.COSPECIATION, cospeciations);
		this.events.put(EventConstants.SORTING, sortings);
		this.events.put(EventConstants.DUPLICATION, duplications);
		this.events.put(EventConstants.HOSTSWITCH, hostswitches);
	}

	public EventType(EventType eventType) {
		this.events = new HashMap<String, Integer>();
		this.events.put(EventConstants.COSPECIATION, new Integer(eventType
				.getCospeciations()));
		this.events.put(EventConstants.SORTING, new Integer(eventType
				.getSortings()));
		this.events.put(EventConstants.DUPLICATION, new Integer(eventType
				.getDuplications()));
		this.events.put(EventConstants.HOSTSWITCH, new Integer(eventType
				.getHostswitches()));
	}

	/**
	 * @return the cospeciations
	 */
	public int getCospeciations() {
		return this.events.get(EventConstants.COSPECIATION);
	}

	/**
	 * @param cospeciations
	 *            the cospeciations to set
	 */
	public void setCospeciations(int cospeciations) {
		this.events.put(EventConstants.COSPECIATION, cospeciations);
	}

	/**
	 * @return the sortings
	 */
	public int getSortings() {
		return this.events.get(EventConstants.SORTING);
	}

	/**
	 * @param sortings
	 *            the sortings to set
	 */
	public void setSortings(int sortings) {
		this.events.put(EventConstants.SORTING, sortings);
	}

	/**
	 * @return the duplications
	 */
	public int getDuplications() {
		return this.events.get(EventConstants.DUPLICATION);
	}

	/**
	 * @param duplications
	 *            the duplications to set
	 */
	public void setDuplications(int duplications) {
		this.events.put(EventConstants.DUPLICATION, duplications);
	}

	/**
	 * @return the hostswitches
	 */
	public int getHostswitches() {
		return this.events.get(EventConstants.HOSTSWITCH);
	}

	/**
	 * @param hostswitches
	 *            the hostswitches to set
	 */
	public void setHostswitches(int hostswitches) {
		this.events.put(EventConstants.HOSTSWITCH, hostswitches);
	}

	/**
	 * @return the events
	 */
	public Map<String, Integer> getEvents() {
		return events;
	}

	/**
	 * @param events
	 *            the events to set
	 */
	public void setEvents(Map<String, Integer> events) {
		this.events = events;
	}

	/**
	 * @param events
	 *            the events to set
	 */
	public void setEvent(String key, Integer value) {
		if (EventConstants.getEvents().contains(key)) {
			this.events.put(key, value);
		}
	}

	public int getEventSum() {
		int sum = 0;
		for (Integer event : this.events.values()) {
			sum += event;
		}
		return sum;
	}

	public EventType add(EventType events) {
		return new EventType(getCospeciations() + events.getCospeciations(),
				getSortings() + events.getSortings(), getDuplications()
						+ events.getDuplications(), getHostswitches()
						+ events.getHostswitches());
	}

	/**
	 * @return a copy of the EventType object
	 */
	public EventType copy() {
		return new EventType(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((events == null) ? 0 : events.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof EventType))
			return false;
		EventType other = (EventType) obj;
		if (events == null) {
			if (other.events != null)
				return false;
		} else if (!events.equals(other.events))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	
	public String toString() {
		return "(" + this.getCospeciations() + " " + this.getSortings() + " "
				+ this.getDuplications() + " " + this.getHostswitches() + ")";
	}
}
