/**
 * 
 */
package com.wieseke.cptk.reconstruction.dao;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ReconstructionElement {

	/**
	 * the host index
	 */
	private int parasiteIndex;
	/**
	 * the host index
	 */
	private int hostIndex;
	/**
	 * the costs of the whole reconstruction
	 */
	private BigDecimal costs;
	/**
	 * the events occurring by mapping only the current parasite
	 */
	private EventType directEvents;
	/**
	 * list of sums of all events including the direct events and the events
	 * needed for the mapping of the children
	 */
	private List<EventType> derivedEvents;
	/**
	 * indices of the host nodes to where the children are mapped on
	 */
	private ReconstructionChildMapping[] childMappings;

	/**
	 * the reconstructions of the child parasites
	 */
	private ReconstructionInfo[] childReconstructionInfos;

	/**
	 * default constructor
	 * 
	 * @param children
	 *            number of child nodes of the current parasite node
	 */
	public ReconstructionElement(int parasiteIndex, int hostIndex, int children) {
		this.parasiteIndex = parasiteIndex;
		this.hostIndex = hostIndex;
		this.costs = null;
		this.directEvents = new EventType(0, 0, 0, 0);
		this.derivedEvents = new ArrayList<EventType>();
		if (children > 0) {
			this.childMappings = new ReconstructionChildMapping[children];
			this.childReconstructionInfos = new ReconstructionInfo[children];
		}
		for (int index = 0; index < children; index++) {
			this.childMappings[index] = new ReconstructionChildMapping();
		}
	}

	/**
	 * @return the parasiteIndex
	 */
	public int getParasiteIndex() {
		return parasiteIndex;
	}

	/**
	 * @param parasiteIndex
	 *            the parasiteIndex to set
	 */
	public void setParasiteIndex(int parasiteIndex) {
		this.parasiteIndex = parasiteIndex;
	}

	/**
	 * @return the hostIndex
	 */
	public int getHostIndex() {
		return hostIndex;
	}

	/**
	 * @param hostIndex
	 *            the hostIndex to set
	 */
	public void setHostIndex(int hostIndex) {
		this.hostIndex = hostIndex;
	}

	/**
	 * @return the costs
	 */
	public BigDecimal getCosts() {
		return costs;
	}

	/**
	 * @param costs
	 *            the costs to set
	 */
	public void setCosts(BigDecimal costs) {
		this.costs = costs;
	}

	/**
	 * @return the directEvents
	 */
	public EventType getDirectEvents() {
		return directEvents;
	}

	/**
	 * @param directEvents
	 *            the directEvents to set
	 */
	public void setDirectEvents(EventType directEvents) {
		this.directEvents = directEvents;
	}

	/**
	 * @return the derivedEvents
	 */
	public List<EventType> getDerivedEvents() {
		return derivedEvents;
	}

	/**
	 * @param derivedEvents
	 *            the derivedEvents to set
	 */
	public void setDerivedEvents(List<EventType> derivedEvents) {
		this.derivedEvents = derivedEvents;
	}

	/**
	 * @return the childMappings
	 */
	public ReconstructionChildMapping[] getChildMappings() {
		return childMappings;
	}

	/**
	 * @param childMappings
	 *            the childMappings to set
	 */
	public void setChildMappings(ReconstructionChildMapping[] childMappings) {
		this.childMappings = childMappings;
	}

	/**
	 * @return the childReconstructions
	 */
	public ReconstructionInfo[] getChildReconstructionInfos() {
		return childReconstructionInfos;
	}

	/**
	 * @param childReconstructions
	 *            the childReconstructions to set
	 */
	public void setChildReconstructions(
			ReconstructionInfo[] childReconstructionInfos) {
		this.childReconstructionInfos = childReconstructionInfos;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	
	public String toString() {
		return this.derivedEvents.toString();
	}
}
