/**
 * 
 */
package com.wieseke.cptk.reconstruction.dao;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import com.wieseke.cptk.common.api.IParasiteNode;
import com.wieseke.cptk.common.dao.ParasiteRank;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ReconstructionParasiteNode extends ReconstructionNode implements
		IParasiteNode {

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.dao.Node#getParent()
	 */
	
	public ReconstructionParasiteNode getParent() {
		return (ReconstructionParasiteNode) super.getParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.reconstruction.dao.ReconstructionNode#getChildren()
	 */
	@SuppressWarnings("unchecked")
	
	public List<ReconstructionParasiteNode> getChildren() {
		return (List<ReconstructionParasiteNode>) super.getChildren();
	}

	/**
	 * timezone information for the parasite node
	 */
	private ParasiteRank rank;

	/**
	 * the ids of the host nodes to which this parasite is mapped on.
	 */
	private List<ReconstructionHostNode> associations;

	/**
	 * the next parasite node in the same level
	 */
	private ReconstructionParasiteNode next;

	/**
	 * the previous parasite node in the same level
	 */
	private ReconstructionParasiteNode prev;

	/**
	 * array of information about the reconstructions, one for each mapping to a
	 * host node
	 */
	private ReconstructionInfo[] reconstructionInfo;

	public boolean[] chronology;

	/**
	 * default constructor
	 */
	public ReconstructionParasiteNode(IParasiteNode node) {
		super(node);
		this.setRank(node.getRank());
		this.associations = new ArrayList<ReconstructionHostNode>();
	}

	/**
	 * initializes the cost and event type arrays
	 * 
	 * @param hostNodesCount
	 */
	public void init(int hostNodesCount) {
		this.reconstructionInfo = new ReconstructionInfo[hostNodesCount];
		this.chronology = new boolean[hostNodesCount];
		for (int hostIndex = 0; hostIndex < hostNodesCount; hostIndex++) {
			this.chronology[hostIndex] = true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IParasiteNode#getRank()
	 */
	
	public ParasiteRank getRank() {
		return this.rank;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.common.api.IParasiteNode#setRank(com.wieseke.cptk.common
	 * .dao.ParasiteRank)
	 */
	
	public void setRank(ParasiteRank rank) {
		this.rank = rank;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IParasiteNode#getMappings()
	 */
	
	public List<ReconstructionHostNode> getAssociations() {
		return this.associations;
	}

	/**
	 * @param associations
	 *            the associations to set
	 */
	public void setAssociations(List<ReconstructionHostNode> associations) {
		this.associations = associations;
	}

	/**
	 * @return the next
	 */
	public ReconstructionParasiteNode getNext() {
		return next;
	}

	/**
	 * @param next
	 *            the next to set
	 */
	public void setNext(ReconstructionParasiteNode next) {
		this.next = next;
	}

	/**
	 * @return the prev
	 */
	public ReconstructionParasiteNode getPrev() {
		return prev;
	}

	/**
	 * @param prev
	 *            the prev to set
	 */
	public void setPrev(ReconstructionParasiteNode prev) {
		this.prev = prev;
	}

	/**
	 * @return the reconstructionInfo
	 */
	public ReconstructionInfo[] getReconstructionInfo() {
		return reconstructionInfo;
	}

	/**
	 * @param reconstructionInfo
	 *            the reconstructionInfo to set
	 */
	public void setReconstructionInfo(int hostNodeIndex,
			ReconstructionInfo reconstructionInfo) {
		this.reconstructionInfo[hostNodeIndex] = reconstructionInfo;
	}

	public BigDecimal getCosts(int hostNodeIndex) {
		if (hostNodeIndex >= 0
				&& hostNodeIndex <= this.reconstructionInfo.length - 1)
			return this.reconstructionInfo[hostNodeIndex].getCosts();
		return null;
	}

}
