/**
 * 
 */
package com.wieseke.cptk.reconstruction.dao;

import java.math.BigDecimal;

import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.common.util.BigDecimalUtil;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ReconstructionSolution {

	/**
	 * the host node index on which the parasite root node is mapped on
	 */
	private Integer rootMapping;
	/**
	 * the total costs for the reconstruction
	 */
	private BigDecimal totalCosts;
	/**
	 * the used cost vector
	 */
	private ReconstructionCosts costs;
	/**
	 * the computed quality value
	 */
	private BigDecimal quality;
	/**
	 * the occurred events
	 */
	private EventType events;

	/**
	 * default constructor
	 * 
	 * @param rootMapping
	 * @param totalCosts
	 * @param costs
	 * @param quality
	 * @param events
	 */
	public ReconstructionSolution(int rootMapping, BigDecimal totalCosts,
			ReconstructionCosts costs, BigDecimal quality, EventType events) {
		this.rootMapping = new Integer(rootMapping);
		this.totalCosts = BigDecimalUtil.copy(totalCosts);
		this.costs = costs.copy();
		this.quality = BigDecimalUtil.copy(quality);
		this.events = events.copy();

	}

	/**
	 * @return the rootMapping
	 */
	public Integer getRootMapping() {
		return rootMapping;
	}

	/**
	 * @param rootMapping
	 *            the rootMapping to set
	 */
	public void setRootMapping(Integer rootMapping) {
		this.rootMapping = rootMapping;
	}

	/**
	 * @return the totalCosts
	 */
	public BigDecimal getTotalCosts() {
		return totalCosts;
	}

	/**
	 * @param totalCosts
	 *            the totalCosts to set
	 */
	public void setTotalCosts(BigDecimal totalCosts) {
		this.totalCosts = totalCosts;
	}

	/**
	 * @return the costs
	 */
	public ReconstructionCosts getCosts() {
		return costs;
	}

	/**
	 * @return the costs
	 */
	public double[] getDoubleCosts() {
		double[] result = new double[4];
		result[0] = getCosts().getCospeciationCosts().doubleValue();
		result[1] = getCosts().getSortingCosts().doubleValue();
		result[2] = getCosts().getDuplicationCosts().doubleValue();
		result[3] = getCosts().getHostswitchCosts().doubleValue();
		return result;
	}

	/**
	 * @param costs
	 *            the costs to set
	 */
	public void setCosts(ReconstructionCosts costs) {
		this.costs = costs;
	}

	/**
	 * @return the quality
	 */
	public BigDecimal getQuality() {
		return quality;
	}

	/**
	 * @param quality
	 *            the quality to set
	 */
	public void setQuality(BigDecimal quality) {
		this.quality = quality;
	}

	/**
	 * @return the events
	 */
	public EventType getEvents() {
		return events;
	}

	/**
	 * @param events
	 *            the events to set
	 */
	public void setEvents(EventType events) {
		this.events = events;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */

	public String toString() {
		return this.quality.toPlainString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((events == null) ? 0 : events.hashCode());
		result = prime * result
				+ ((rootMapping == null) ? 0 : rootMapping.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ReconstructionSolution other = (ReconstructionSolution) obj;
		if (events == null) {
			if (other.events != null)
				return false;
		} else if (!events.equals(other.events))
			return false;
		if (rootMapping == null) {
			if (other.rootMapping != null)
				return false;
		} else if (!rootMapping.equals(other.rootMapping))
			return false;
		return true;
	}

}
