/**
 * 
 */
package com.wieseke.cptk.statistics;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import com.wieseke.cptk.common.api.IPositionNode;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;

/**
 * @author Nicolas Wieseke
 * 
 */
public class AssociationGenerator {

	/**
	 * @param cophylogeny
	 */
	public static void generateRandomAssociations(InputCophylogeny cophylogeny,
			Random random) {
		if (random == null) {
			random = new Random();
		}
		List<InputHostNode> hostLeaves = new ArrayList<InputHostNode>();
		List<InputParasiteNode> parasiteLeaves = new ArrayList<InputParasiteNode>();
		for (IPositionNode hostNode : cophylogeny.getNodeList(cophylogeny
				.getHostRoot())) {
			if (hostNode.isLeaf())
				hostLeaves.add((InputHostNode) hostNode);
		}
		for (IPositionNode parasiteNode : cophylogeny.getNodeList(cophylogeny
				.getParasiteRoot())) {
			if (parasiteNode.isLeaf())
				parasiteLeaves.add((InputParasiteNode) parasiteNode);
		}
		for (InputParasiteNode parasite : parasiteLeaves) {
			parasite.setAssociations(new ArrayList<InputHostNode>());
			int index = random.nextInt(hostLeaves.size());
			parasite.addAssociations(hostLeaves.get(index));
		}
	}

	/**
	 * @param hostLeaves
	 * @param parasiteLeaves
	 */
	public static void createMultipleAssociations(
			List<InputHostNode> hostLeaves,
			List<InputParasiteNode> parasiteLeaves, Random random) {
		if (random == null) {
			random = new Random();
		}
		for (InputParasiteNode parasite : parasiteLeaves) {
			int index = random.nextInt(hostLeaves.size());
			parasite.addAssociations(hostLeaves.get(index));
		}
	}

	/**
	 * @param hostLeaves
	 * @param parasiteLeaves
	 */
	public static void createUniqueAssociations(List<InputHostNode> hostLeaves,
			List<InputParasiteNode> parasiteLeaves, Random random) {
		if (random == null) {
			random = new Random();
		}
		List<InputHostNode> hostLeavesCopy = new ArrayList<InputHostNode>();
		hostLeavesCopy.addAll(hostLeaves);
		List<InputParasiteNode> parasiteLeavesCopy = new ArrayList<InputParasiteNode>();
		parasiteLeavesCopy.addAll(parasiteLeaves);
		while (hostLeavesCopy.size() > 0 && parasiteLeavesCopy.size() > 0) {
			int hostIndex = random.nextInt(hostLeavesCopy.size());
			int parasiteIndex = random.nextInt(parasiteLeavesCopy.size());
			parasiteLeavesCopy.get(parasiteIndex).addAssociations(
					hostLeavesCopy.get(hostIndex));
			hostLeavesCopy.remove(hostIndex);
			parasiteLeavesCopy.remove(parasiteIndex);
		}
	}

	/**
	 * @param originalHostLeaves
	 * @param originalParasiteLeaves
	 * @param hostLeaves
	 * @param parasiteLeaves
	 */
	public static void createOriginalAssociations(
			ArrayList<InputHostNode> originalHostLeaves,
			ArrayList<InputParasiteNode> originalParasiteLeaves,
			List<InputHostNode> hostLeaves,
			List<InputParasiteNode> parasiteLeaves) {
		if (originalHostLeaves.size() != hostLeaves.size()
				|| originalParasiteLeaves.size() != parasiteLeaves.size()) {
			return;
		}
		for (int i = 0; i < originalParasiteLeaves.size(); i++) {
			InputParasiteNode originalParasiteLeaf = originalParasiteLeaves
					.get(i);
			InputParasiteNode parasiteLeaf = parasiteLeaves.get(i);
			for (InputHostNode mappingPartner : originalParasiteLeaf
					.getAssociations()) {
				int index = originalHostLeaves.indexOf(mappingPartner);
				if (index >= 0) {
					parasiteLeaf.getAssociations().add(hostLeaves.get(index));
				}
			}
		}
	}

	/**
	 * @param originalHostLeaves
	 * @param originalParasiteLeaves
	 * @param hostLeaves
	 * @param parasiteLeaves
	 */
	public static void createRandomAssociationsWithOriginalCharacteristics(
			List<InputHostNode> originalHostLeaves,
			List<InputParasiteNode> originalParasiteLeaves,
			List<InputHostNode> hostLeaves,
			List<InputParasiteNode> parasiteLeaves, Random random) {
		if (random == null) {
			random = new Random();
		}
		if (originalHostLeaves.size() != hostLeaves.size()
				|| originalParasiteLeaves.size() != parasiteLeaves.size()) {
			return;
		}
		originalHostLeaves = permuteHostNodes(originalHostLeaves, random);
		for (int i = 0; i < originalParasiteLeaves.size(); i++) {
			InputParasiteNode originalParasiteLeaf = originalParasiteLeaves
					.get(i);
			InputParasiteNode parasiteLeaf = parasiteLeaves.get(i);
			for (InputHostNode mappingPartner : originalParasiteLeaf
					.getAssociations()) {
				int index = originalHostLeaves.indexOf(mappingPartner);
				if (index >= 0) {
					parasiteLeaf.getAssociations().add(hostLeaves.get(index));
				}
			}
		}
	}

	private static List<InputHostNode> permuteHostNodes(
			List<InputHostNode> nodes, Random random) {
		List<InputHostNode> permutatedNodes = new ArrayList<InputHostNode>();
		while (nodes.size() > 0) {
			int index = random.nextInt(nodes.size());
			permutatedNodes.add(nodes.get(index));
			nodes.remove(index);
		}
		return permutatedNodes;
	}
}
